-- Lua Wireshark Dissector of ULPFEC (RFC 5109)
-- Author: Camille GONNET (camille@sound4.biz)
--
-- This Dissector is distributed under the GNU General Public License v2.0
-- in the hope that it will be of use but WITHOUT ANY WARRENTY; without even
-- the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
--
-- See the GNU General Public License for more details.

-- requires liblua bitop http://bitop.luajit.org
-- included in Wireshark now
-- local bit = require("bit")
local function bext(value, bitoffset, bitlen)
  return bit.band( bit.rshift(value, bitoffset) , bit.lshift(1,bitlen)-1)
end
local function swap16(value)
  return bit.rshift( bit.bswap( value ), 16)
end

ULPFEC = Proto ("ULPFEC","ULPFEC - RFC 5109")

local f = ULPFEC.fields

-- FEC Header
f.Hdr_flags = ProtoField.uint16("ulpfec.header.flags","FEC Header flags",base.HEX,nil,0xffff)
    f.Hdr_E = ProtoField.uint16("ulpfec.header.flags.E", "extension", base.HEX, nil, 0x8000)
    f.Hdr_L = ProtoField.uint16("ulpfec.header.flags.L", "long mask", base.HEX, nil, 0x4000)
    f.Hdr_P = ProtoField.uint16("ulpfec.header.flags.P", "P recovery", base.HEX, nil, 0x2000)
    f.Hdr_X = ProtoField.uint16("ulpfec.header.flags.X", "X recovery", base.HEX, nil, 0x1000)
    f.Hdr_CC = ProtoField.uint16("ulpfec.header.flags.CC", "CC recovery", base.HEX, nil, 0x0F00)
    f.Hdr_M = ProtoField.uint16("ulpfec.header.flags.M", "M recovery", base.HEX, nil, 0x0080)
    f.Hdr_PT = ProtoField.uint16("ulpfec.header.flags.PT", "PT recovery", base.HEX, nil, 0x007F)
f.Hdr_sn = ProtoField.uint16("ulpfec.header.snbase","FEC Header SN Base",base.DEC)
f.Hdr_ts = ProtoField.uint32("ulpfec.header.tsrec","FEC Header TS rec",base.HEX)
f.Hdr_len = ProtoField.uint16("ulpfec.header.lenrec","FEC Header Len rec",base.HEX)

-- FEC Level Header
f.LvlHdr_len = ProtoField.uint16("ulpfec.levelheader.len","Protection Length",base.DEC)
f.LvlHdr_mask = ProtoField.uint16("ulpfec.levelheader.mask","Protection mask",base.HEX)
f.LvlHdr_longmask = ProtoField.uint64("ulpfec.levelheader.longmask","Protection mask (long)",base.HEX, nil, 0xFFFFFFFFFFFF)

f.Payload = ProtoField.bytes("ulpfec.payload","FEC")

function ULPFEC.dissector (buffer, packet, root)
    if buffer:len() == 0 then return end
    packet.cols.protocol = ULPFEC.name

    fec = root:add(ULPFEC, buffer(0))

    hdr = fec:add("FEC Header", buffer(0,10))
    hdr_flags = hdr:add(f.Hdr_flags, buffer(0,2))
        hdr_flags:add(f.Hdr_E, buffer(0,2))
        hdr_flags:add(f.Hdr_L, buffer(0,2))
        hdr_flags:add(f.Hdr_P, buffer(0,2))
        hdr_flags:add(f.Hdr_X, buffer(0,2))
        hdr_flags:add(f.Hdr_CC, buffer(0,2))
        hdr_flags:add(f.Hdr_M, buffer(0,2))
        hdr_flags:add(f.Hdr_PT, buffer(0,2))
    hdr:add(f.Hdr_sn, buffer(2,2))
    hdr:add(f.Hdr_ts, buffer(4,4))
    hdr:add(f.Hdr_len, buffer(8,2))

    longmask = bext(buffer(0,1):uint(),6,1)
    firstsn = buffer(2,2):uint()
    levelnum = 0
    leveloffset = 10
    while leveloffset < buffer:len() do
        sn = firstsn
        snlist = nil
        if (longmask==0) then
            lvl = fec:add("Level " .. levelnum .. " Header", buffer(leveloffset,4))
            lvl:add(f.LvlHdr_len, buffer(leveloffset,2))
            lvl:add(f.LvlHdr_mask, buffer(leveloffset+2,2))
            mask = buffer(leveloffset+2,2):uint()
            pl_offset=leveloffset+4

            for i=0,15 do
                if (bext(mask,15-i,1)==1) then
                    if snlist == nil then
                        snlist = tostring(sn)
                    else
                        snlist = snlist .. "," .. tostring(sn)
                    end
                end
                sn=sn+1
            end
            lvl:append_text(" (SN:" .. snlist .. ")")
        else
            lvl = fec:add("Level " .. levelnum .. " Header", buffer(leveloffset,8))
            lvl:add(f.LvlHdr_len, buffer(leveloffset,2))
            lvl:add(f.LvlHdr_longmask, buffer(leveloffset+2,6))
            pl_offset=leveloffset+8

            mask = buffer(leveloffset+2,2):le_uint()
            for i=0,15 do
                if (bext(mask,15-i,1)==1) then
                    if snlist == nil then
                        snlist = tostring(sn)
                    else
                        snlist = snlist .. "," .. tostring(sn)
                    end
                end
                sn=sn+1
            end
            mask = buffer(14,4):le_uint()
            for i=0,31 do
                if (bext(mask,31-i,1)==1) then
                    if snlist == nil then
                        snlist = tostring(sn)
                    else
                        snlist = snlist .. "," .. tostring(sn)
                    end
                end
                sn=sn+1
            end
            lvl:append_text(" (SN:" .. snlist .. ")")
        end

        protection_length = buffer(leveloffset,2):uint()

        pl = fec:add("Level " .. levelnum .. " payload (Size " .. protection_length .. ")", buffer(pl_offset,protection_length))
        pl:add(f.Payload, buffer(pl_offset,protection_length))

        leveloffset = pl_offset + protection_length
        levelnum = levelnum +1
    end
    if levelnum == 1 then
        -- single level
        packet.cols.info = ULPFEC.name .. " for SN:" .. snlist .. " protect_len=" .. protection_length
    else
        packet.cols.info = ULPFEC.name .. " Level 0-" .. (levelnum-1) .. " starting on SN:" .. firstsn .. " total_len=" .. buffer:len()
    end
end

function ULPFEC.init()
end

local rtp_dissector_table = DissectorTable.get("rtp.pt")
dissector = rtp_dissector_table:get_dissector(96)
rtp_dissector_table:add(98, ULPFEC)
